#!/usr/bin/perl
# Replicate configs to all slaves

package server_manager;
$main::no_acl_check++;
$ENV{'WEBMIN_CONFIG'} ||= "/etc/webmin";
$ENV{'WEBMIN_VAR'} ||= "/var/webmin";
if ($0 =~ /^(.*)\/[^\/]+$/) {
	chdir($pwd = $1);
	}
else {
	chop($pwd = `pwd`);
	}
$0 = "$pwd/replicate.pl";
require './server-manager-lib.pl';
$< == 0 || die "$0 must be run as root";

# Parse command line
@servers = ( );
while(@ARGV) {
	$a = shift(@ARGV);
	if ($a eq "--debug") {
		$debug = 1;
		}
	elsif ($a eq "--dry-run") {
		$dryrun = 1;
		}
	elsif ($a eq "--host") {
		$hostname = shift(@ARGV);
		$server = &get_managed_server_by("host", $hostname);
		$server || &usage("No system with hostname $hostname exists");
		push(@servers, $server);
		}
	else {
		&usage();
		}
	}
if (&get_replication_mode() != 2) {
	&usage("Not a Cloudmin replication master");
	}

# Replicate to all slaves
if (@servers) {
	@rv = ( );
	foreach my $s (@servers) {
		push(@rv, &replicate_host_to_all_servers($s, $dryrun));
		}
	}
else {
	@rv = &replicate_to_servers($dryrun);
	}

# Show what was done
if ($debug) {
	foreach my $r (@rv) {
		print $r->{'replica'}->{'host'},"\n";
		print "    Status: ",$r->{'ok'} ? "OK" : "Failed","\n";
		if (!$r->{'ok'}) {
			print "    Error: $r->{'err'}\n";
			}
		foreach my $s (@{$r->{'servers'}}) {
			print "    System: $s->{'host'}\n";
			}
		foreach my $f (@{$r->{'copied'}}) {
			print "    Copied: $f\n";
			}
		foreach my $f (@{$r->{'deleted'}}) {
			print "    Deleted: $f\n";
			}
		}
	}

sub usage
{
print "$_[0]\n\n" if ($_[0]);
print "Replicates Cloudmin configurations to all replicas.\n";
print "\n";
print "cloudmin replicate [--debug]\n";
print "                   [--dry-run]\n";
print "                   [--host hostname]*\n";
exit(1);
}

